@@ -1,8 +1,3 @@
-#paste(as.character(Call$weights), collapse="")
-#oppure
-#as.character(Call$weights)[1]
-#comunque poi il problema  come aggiornare la componente weights della call objF$weights
-
 `segmented.glm` <-
 #objF$id.group???
 function(obj, seg.Z, psi=stop("provide psi"), control = seg.control(), model = TRUE, ...) {
@@ -49,14 +44,15 @@ function(obj, seg.Z, psi=stop("provide psi"), control = seg.control(), model = T
 #    a <- subset(a, select = colnames(a)[-1])
     orig.call<-Call<-mf<-obj$call
     orig.call$formula<-mf$formula<-formula(obj) #per consentire lm(y~.)
-    m <- match(c("formula", "data", "subset", "weights", "na.action","offset"), names(mf), 0L)
+    m <- match(c("formula", "data", "subset", "weights", "na.action","etastart","mustart","offset"), names(mf), 0L)
     mf <- mf[c(1, m)]
     mf$drop.unused.levels <- TRUE
     mf[[1L]] <- as.name("model.frame")
+    #non so a che serva la seguente linea..
     if(class(mf$formula)=="name" && !"~"%in%paste(mf$formula)) mf$formula<-eval(mf$formula)
     #orig.call$formula<-update.formula(orig.call$formula, paste("~.-",all.vars(seg.Z))) #utile per plotting
     
-    nomeRispo<-strsplit(paste(formula(obj))[2],"/")[[1]] #eventuali doppi nomi (tipo "y/n" per GLM binom)
+#    nomeRispo<-strsplit(paste(formula(obj))[2],"/")[[1]] #eventuali doppi nomi separati da "/" (tipo "y/n" per GLM binom)
     #la linea sotto aggiunge nel mf anche la variabile offs..
 #    if(length(all.vars(formula(obj)))>1){
 #      id.rispo<-1
@@ -68,43 +64,67 @@ function(obj, seg.Z, psi=stop("provide psi"), control = seg.control(), model = T
     #  mf$formula<-update.formula(mf$formula,paste(seg.Z,collapse=".+"))
 #    }
     mfExt<- mf
-    mfExt$formula<- update.formula(mfExt$formula,paste(paste(seg.Z,collapse=".+"),"+",paste(all.vars(formula(obj)),collapse="+")))   
+    mf$formula<-update.formula(mf$formula,paste(seg.Z,collapse=".+"))
+
+#    mfExt$formula<- update.formula(mfExt$formula,paste(paste(seg.Z,collapse=".+"),"+",paste(all.vars(formula(obj)),collapse="+")))   
+#    mfExt$formula<- if(!is.null(obj$call$data)) 
+#        update.formula(mf$formula,paste(".~",paste(all.vars(obj$call), collapse="+"),"-",obj$call$data,sep=""))
+#            else update.formula(mf$formula,paste(".~",paste(all.vars(obj$call), collapse="+"),sep=""))
+#-----------
+   if(!is.null(obj$call$offset) || !is.null(obj$call$weights) || !is.null(obj$call$subset)){ 
+          mfExt$formula<-update.formula(mf$formula,paste(".~.+", c(all.vars(obj$call$offset),  
+          all.vars(obj$call$weights), all.vars(obj$call$subset)) , collapse="+"),sep="")
+          }
+
+    mf <-  eval(mf, parent.frame())
+    #La linea sotto serve per inserire in mfExt le eventuali variabili contenute nella formula con offset(..)
+    #   o anche variabili che rientrano in espressioni (ad es., y/n o I(y*n))
+    nomiOff<-setdiff(all.vars(formula(obj)), names(mf))
+    if(length(nomiOff)>=1) mfExt$formula<-update.formula(mfExt$formula,paste(".~.+", paste( nomiOff, collapse="+"), sep=""))
     mfExt<-eval(mfExt, parent.frame())
 
-    mf$formula<-update.formula(mf$formula,paste(seg.Z,collapse=".+"))
-    mf <- eval(mf, parent.frame())
     #id.offs<-pmatch("offset",names(mf)) #questa identifica il nome offset(..). ELiminarlo dal dataframe? non conviene
     #       altrimenti nel model.frame non risulta l'offset
 
     #mantieni in mfExt solo le variabili che NON ci sono in mf (cos la funzione occupa meno spazio..)
-    mfExt<-mfExt[,setdiff(names(mfExt), names(mf)),drop=FALSE]
+    #mfExt<-mfExt[,setdiff(names(mfExt), names(mf)),drop=FALSE]
 
     weights <- as.vector(model.weights(mf))
     offs <- as.vector(model.offset(mf))
     
-    if(!is.null(Call$weights)){ #"(weights)"%in%names(mf)
-      names(mf)[which(names(mf)=="(weights)")]<-all.vars(Call$weights, functions=FALSE) #as.character(Call$weights) #paste(as.character(Call$weights), collapse="")
-      # mf["(weights)"]<-weights
-      }
-    
+#    if(!is.null(Call$weights)){ #"(weights)"%in%names(mf)
+#      mfExt[all.vars(Call$weights, functions=FALSE)]<- eval(parse(text=all.vars(Call$weights)))
+#      #names(mfExt)[which(names(mf)=="(weights)")]<- all.vars(Call$weights, functions=FALSE)#prima di 0.2.9-4 era as.character(Call$weights)
+#      # mf["(weights)"]<-weights
+#      }
+#
+#    if(!is.null(Call$offset)){ 
+#     mfExt[all.vars(Call$offset, functions=FALSE)]<- eval(parse(text=all.vars(Call$offset)))
+##     mfExt["(offset)"]<-eval(parse(text=all.vars(Call$offset)))
+##      names(mf)[which(names(mf)=="(offset)")]<-all.vars(Call$offset, functions=FALSE) 
+#     }
+
     mt <- attr(mf, "terms")
     interc<-attr(mt,"intercept")
     y <- model.response(mf, "any")
     XREG <- if (!is.empty.model(mt)) model.matrix(mt, mf, contrasts)
     
     #il cambio in mf da "offset(_nomevar_)" al "_nomevar_" deve avvenire dopo "model.matrix(mt, mf, contrasts)" 
-    if(!is.null(offs)){
-      id.offs<-pmatch("offset",names(mf)) #questa identifica il nome offset(..). ELiminarlo dal dataframe? non conviene altrimenti nel model.frame non risulta l'offset
-      names(mf)[id.offs]<- all.vars(formula(paste("~", names(mf)[id.offs])), functions=FALSE)
-      }
+#    if(!is.null(offs)){
+#      #id.offs<-pmatch("offset",names(mf)) #questa identifica il nome offset(..). ELiminarlo dal dataframe? non conviene altrimenti nel model.frame non risulta l'offset
+#      id.offs<- which(grepl("(offset)", names(mf))) #per consentire anche offset come argomento di glm()
+#      names(mf)[id.offs]<- all.vars(formula(paste("~", names(mf)[id.offs])), functions=FALSE)
+#      }
     
     namesXREG0<-colnames(XREG)
     #nameLeftSlopeZero<-setdiff(all.vars(seg.Z), all.vars(formula(obj)))
     nameLeftSlopeZero<-setdiff(all.vars(seg.Z), names(coef(obj))) #in questo modo riconosce che sin(x*pi) NON  x, ad esempio.
     namesXREG0<-setdiff(namesXREG0, nameLeftSlopeZero)
     
+    #dalla 0.3.0-1 eliminati i seguenti (tanto il modello viene stimato su mfExt)
+    #nomeRispo<-strsplit(paste(formula(obj))[2],"/")[[1]] #eventuali doppi nomi separati da "/" (tipo "y/n" per GLM binom)
     #nomeRispo<-strsplit(paste(formula(obj))[2],"/")[[1]] #portato sopra
-    if(length(nomeRispo)>=2) mf[nomeRispo[1]]<-weights*y
+    #if(length(nomeRispo)>=2) mf[nomeRispo[1]]<-weights*y
     
     id.duplic<-match(all.vars(formula(obj)),all.vars(seg.Z),nomatch=0)>0
     if(any(id.duplic)) {
@@ -122,6 +142,8 @@ function(obj, seg.Z, psi=stop("provide psi"), control = seg.control(), model = T
     #XREG<-XREG[,colnames(model.matrix(obj)),drop=FALSE]
     #XREG<-XREG[,match(c("(Intercept)",all.vars(formula(obj))[-1]),colnames(XREG),nomatch =0),drop=FALSE]
     XREG <- XREG[, match(c("(Intercept)", namesXREG0),colnames(XREG), nomatch = 0), drop = FALSE]
+    XREG<-XREG[,unique(colnames(XREG)), drop=FALSE]
+    
     n <- nrow(XREG)
     #Z <- list(); for (i in colnames(XREGseg)) Z[[length(Z) + 1]] <- XREGseg[, i]
     Z<-lapply(apply(XREGseg,2,list),unlist) #prende anche i nomi!
@@ -165,7 +187,7 @@ function(obj, seg.Z, psi=stop("provide psi"), control = seg.control(), model = T
 #    KK <- new.env()
 #    for (i in 1:ncol(objframe$model)) assign(names(objframe$model[i]), objframe$model[[i]], envir = KK)
     if (it.max == 0) {
-        mf<-cbind(mf, mfExt)
+        #mf<-cbind(mf, mfExt)
         U <- pmax((Z - PSI), 0)
         colnames(U) <- paste(ripetizioni, nomiZ, sep = ".")
         nomiU <- paste("U", colnames(U), sep = "")
@@ -175,9 +197,9 @@ function(obj, seg.Z, psi=stop("provide psi"), control = seg.control(), model = T
         Fo <- update.formula(formula(obj), as.formula(paste(".~.+", paste(nomiU, collapse = "+"))))
         #obj <- update(obj, formula = Fo, data = KK)
         
-        obj <- update(obj, formula = Fo, data = mf, evaluate=FALSE)
-        if(!is.null(obj[["subset"]])) obj[["subset"]]<-NULL
-        obj<-eval(obj, envir=mf)
+        obj <- update(obj, formula = Fo, data = mfExt, evaluate=FALSE)
+        #if(!is.null(obj[["subset"]])) obj[["subset"]]<-NULL
+        obj<-eval(obj, envir=mfExt)
         if (model) obj$model <-mf  #obj$model <- data.frame(as.list(KK))
         names(psi)<-paste(paste("psi", ripetizioni, sep = ""), nomiZ, sep=".")
         obj$psi <- psi
@@ -241,10 +263,10 @@ function(obj, seg.Z, psi=stop("provide psi"), control = seg.control(), model = T
     nomiU   <- unlist(mapply(forma.nomiU, length.psi, name.Z)) #invece di un ciclo #paste("U",1:length.psi[i], ".", name.Z[i])
     nomiVxb <- unlist(mapply(forma.nomiVxb, length.psi, name.Z))
 
-    mf<-cbind(mf, mfExt)
+    #mf<-cbind(mf, mfExt)
     for(i in 1:ncol(U)) {
-        mf[nomiU[i]]<-U[,i]
-        mf[nomiVxb[i]]<-Vxb[,i]
+        mfExt[nomiU[i]]<-mf[nomiU[i]]<-U[,i]
+        mfExt[nomiVxb[i]]<-mf[nomiVxb[i]]<-Vxb[,i]
         }
 #    for (i in 1:ncol(U)) {
 #        assign(nomiU[i], U[, i], envir = KK)
@@ -253,14 +275,24 @@ function(obj, seg.Z, psi=stop("provide psi"), control = seg.control(), model = T
     nnomi <- c(nomiU, nomiVxb)
     Fo <- update.formula(formula(obj0), as.formula(paste(".~.+", 
         paste(nnomi, collapse = "+"))))
+    #la seguente linea si potrebbe rimuovere perch in mfExt c' gi tutto..
     if(is.matrix(y)&& (fam$family=="binomial" || fam$family=="quasibinomial")){
-              mf<-cbind(mf[[1]], mf[,-1])
+              mfExt<-cbind(mfExt[[1]], mfExt[,-1])
     }
-    objF <- update(obj0, formula = Fo, data = mf, evaluate=FALSE)
-    if(!is.null(objF[["subset"]])) objF[["subset"]]<-NULL
-    objF<-eval(objF, envir=mf)
-
-    #c' un problema..controlla obj (ha due "(Intercepts)"
+    objF <- update(obj0, formula = Fo, data = mfExt, evaluate=FALSE)
+#    if(!is.null(objF[["subset"]])) objF[["subset"]]<-NULL
+    objF<-eval(objF, envir=mfExt)
+    #C' un problema..controlla obj (ha due "(Intercepts)" - bhu.. al 27/03/14 non mi sembra!
+    #Pu capitare che psi sia ai margini e ci sono 1 o 2 osservazioni in qualche intervallo. Oppure ce ne 
+    #   sono di pi ma hanno gli stessi valori di x
+    if(any(is.na(objF$coefficients))){
+     stop("at least one coef estimate is NA: breakpoint(s) at the boundary? (possibly with many x-values replicated)", call. = FALSE)
+    }
+#aggiornare qui i weights???? (piuttosto che sotto)
+#------>>>
+#------>>>
+#------>>>
+    objF$offset<- obj0$offset
     if(!gap){
         names.coef<-names(objF$coefficients)
         if(k==1) {names(obj$coefficients)[match(c("U","V"), names(coef(obj)))]<- nnomi
@@ -273,7 +305,7 @@ function(obj, seg.Z, psi=stop("provide psi"), control = seg.control(), model = T
         objF$linear.predictors<-obj$linear.predictors
         objF$residuals<-obj$residuals
         objF$deviance<-obj$deviance
-        objF$aic<-obj$aic
+        objF$aic<-obj$aic + 2*k
         objF$weights<-obj$weights
         }
     if(any(is.na(objF$coefficients))){
